#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include <string.h>
#include <libxml/parser.h>
#include <libxml/tree.h>
#include <curl/curl.h>
#include <getopt.h>
#include <sys/stat.h>

#include "speedtest.h"

static const char *opt_string = ":c:C:i:p:s:u:d:h";
static struct option long_options[] = {
	{ "country", required_argument, NULL, 'C' },
	{ "city", required_argument, NULL, 'c' },
	{ "provider", required_argument, NULL, 'p' },
	{ "source", required_argument, NULL, 's' },
	{ "id", required_argument, NULL, 'i' },
	{ "ul-size", required_argument, NULL, 'u' },
	{ "dl-size", required_argument, NULL, 'd' },
	{ "help", no_argument, NULL, 'h' },
	{ 0, 0, 0, 0 },
};

static void banner(void)
{
	printf("\n");
	printf(" _______________________________________________________________ \n");
	printf("|    ____                 _                 ____               |\n");
	printf("|   |  _ \\ __ _ _ __   __| | ___  _ __ __ _| __ )  _____  __   |\n");
	printf("|   | |_) / _` | '_ \\ / _` |/ _ \\| '__/ _` |  _ \\ / _ \\ \\/ /   |\n");
	printf("|   |  __/ (_| | | | | (_| | (_) | | | (_| | |_) | (_) >  <    |\n");
	printf("|   |_|   \\__,_|_| |_|\\__,_|\\___/|_|  \\__,_|____/ \\___/_/\\_\\   |\n");
	printf("|                                                              |\n");
	printf("|               PandoraBox SpeedTest client by lintel          |\n");
	printf("|                Copyright 2018 PandoraBox Team                |\n");
	printf("|______________________________________________________________|\n\n");
	 
}

static void usage(void)
{
	printf(
		"    %s, v%s\n"
		"    Usage: %s [OPTIONS]\n"
		"\n"
		"	Options:\n"
		"	-C, --country		choose a server from this country\n"
		"	-c, --city		choose a server from this city\n"
		"	-p, --provider		choose a server from this provider\n"
		"	-s, --source		server config source (file or url)\n"
		"	-i, --id		choose a server with this id\n"
		"	-u, --ul-size		upload size\n"
		"	-d, --dl-size		upload size\n"
		"	-D, --debug		debug\n"
		"	-h, --help		show this help message\n"
		"\n"
		"    Report bugs to %s\n"
		"\n",
		PACKAGE_NAME, PACKAGE_VERSION, PACKAGE_NAME, PACKAGE_BUGREPORT);
}

static void save_speedtest_value(unsigned long down,unsigned long up)
{
	char buf[256];
	sprintf(buf,"echo %lu %lu > /tmp/speedtest",down,up);
	system(buf);
}

int main(int argc, char **argv)
{
	int opt;
	int option_index = 0;
	size_t optlen = 0;
	size_t ulsize = SPEEDTEST_UPLOAD_SIZE;
	size_t dlsize = SPEEDTEST_DOWNLOAD_SIZE;
	speedtest_server_t *server;
	speedtest_client_t *client;
	char *download_url;
	double download_speed;
	double upload_speed;
	int ret = -1;
	
	banner();
	
	while ((opt = getopt_long(argc, argv,
				    opt_string, long_options,
				    &option_index)) != -1) {
		switch(opt) {
			case 'c':
				optlen = strlen(optarg) + 1;
// 			srv.city = (char *)malloc(optlen);
// 			if (!srv.city)
// 				goto exit;
// 			snprintf(srv.city, optlen, "%s", optarg);
				break;

			case 'u':
				ulsize = atol(optarg);
				break;

			case 'd':
				ulsize = atol(optarg);
				break;

			case 'h':
				usage();
				goto exit;;

			default:
				printf("%s: option -%c is invalid\n", argv[0], optopt);
				goto exit;
		}
	}

	ret = get_speedtest_servers_list(SPEEDTEST_SERVER_FILE);
	if(ret < 0)
	{
	    printf("get_speedtest_servers_list error\n");
	     goto exit;
	}
	
	ret = convert_server_xml(SPEEDTEST_SERVER_FILE);
	if(ret < 0)
	{
	    printf("convert_server_xml error\n");
	    goto exit;
	}
	
	ret = get_speedtest_client_info(SPEEDTEST_CLIENT_FILE);
	if(ret < 0)
	{
	    printf("get_speedtest_client_info error\n");
	     goto exit;
	}
	
	client = convert_client_xml(SPEEDTEST_CLIENT_FILE);
	printf("IP:%s ISP:%s Local:[%f,%f]\n", client->ip, client->isp, client->lat, client->lon);

	server = get_best_server_by_distance(client);
	printf("Best Server Name: %s Local:[%f,%f]\nCountry: %s Sponsor: %s URL: %s\n\t Dist: %ld km\n",
	       server->name,
	       server->lat, server->lon,
	       server->country, server->sponsor,
	       server->url, server->distance);

	download_speed = speedtest_download(server->host, ulsize);
	if(download_speed < 0)
	     printf("speedtest_download error\n");
	
	upload_speed = speedtest_upload(server->url, dlsize);
	if(download_speed < 0)
	     printf("speedtest_upload error\n");
	
	printf("Download speed: %.2f Mbps, Upload: %.2f Mbps\n", download_speed, upload_speed);
	
	save_speedtest_value((unsigned long)(download_speed*1024), (unsigned long)(upload_speed * 1024) );
exit:
	return 0;
}
