#ifndef _SPEEDTEST_
#define _SPEEDTEST_
#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include <string.h>
#include <curl/curl.h>
#include <libxml/parser.h>
#include <libxml/tree.h>

#define PACKAGE_NAME		"SpeedTest"
#define PACKAGE_VERSION		"1.0"
#define PACKAGE_BUGREPORT 	"lintel<lintel.huang@gmail.com>"

#define SPEEDTEST_CLIENT_URL	"http://www.speedtest.net/speedtest-config.php"
#define SPEEDTEST_SERVER_URL	"http://www.speedtest.net/speedtest-servers-static.php"

#define SPEEDTEST_SERVER_FILE "/tmp/.speedtest_server_list.xml"
#define SPEEDTEST_CLIENT_FILE "/tmp/.speedtest_client_info.xml"

#define  SPEEDTEST_USERAGENT	"Mozilla/5.0 (Windows NT 10.0; WOW64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/67.0.3396.99 Safari/537.36"

//#define DLFILE		"random4000x4000.jpg"

#define SPEEDTEST_DOWNLOAD_SIZE		(100*1024*1024)
#define SPEEDTEST_UPLOAD_SIZE		(30*1024*1024)

unsigned long download_speed;
unsigned long upload_speed;

#define R 6371
#define PI 3.1415926536
#define TO_RAD (PI / 180)

#ifndef ARRAY_SIZE
#define ARRAY_SIZE(_x) \
	(sizeof(_x) / sizeof(_x[0]))
#endif /* ARRAY_SIZE */

#define DIR_DOWNLOAD	0
#define DIR_UPLOAD		1
#define PROGRESS_INT	2

typedef struct mem_block {
	char *chunk;
	size_t size;
} mem_block_t;

typedef struct progress {
	int direction;
	double max_speed;
	double avg_speed;
	double min_speed;
	double lastruntime;
	size_t last;
	int done;
	CURL *curl;
} progress_t;

typedef struct speedtest_client {
	char *ip;
	float lat;
	float lon;
	char *isp;
} speedtest_client_t;

typedef struct speedtest_server {
	char *url;
	char *url2;
	char *id;
	float lat;
	float lon;
	char *name;
	char *country;
	char *cc;
	char *sponsor;
	char *host;
	union {
		long distance;
		long ping;
	};
	struct speedtest_server *next;
} speedtest_server_t;

speedtest_server_t **server_list;
unsigned int server_count;

int convert_server_xml(char *xml_file);
speedtest_client_t *convert_client_xml(char *xml_file);

int get_speedtest_servers_list(const char *file);
int get_speedtest_client_info(const char *file);

void get_start_time(struct timeval *tval_start);
float get_elapsed_time(struct timeval tval_start);

speedtest_server_t *get_best_server_by_distance(speedtest_client_t *client);

double speedtest_download(const char *host, size_t size);
double speedtest_upload(const char *url, size_t size);

#endif
