#include <stdio.h>
#include <string.h>
#include <libxml/parser.h>
#include <libxml/tree.h>
#include <curl/curl.h>
#include <getopt.h>
#include <sys/stat.h>

#include "speedtest.h"

#if 0
static size_t write_mem(void *curl_buf, size_t size,
			 size_t nmemb, void *userp)
{
	size_t curl_size = size * nmemb;
	mem_block_t *mem = (mem_block_t *)userp;

	if (!mem->chunk) {
		mem->size += curl_size;
		return curl_size;
	}

	mem->chunk = (char *)realloc(mem->chunk, mem->size + curl_size + 1);

	if (!mem->chunk)
		return 0;

	memcpy(&(mem->chunk[ mem->size ]), curl_buf, curl_size);
	mem->size += curl_size;
	mem->chunk[ mem->size ] = 0;

	return curl_size;
}

static size_t read_mem(void *curl_buf, size_t size,
			size_t nmemb, void *userp)
{
	size_t curl_size = nmemb * size;
	mem_block_t *mem = (mem_block_t *)userp;
	size_t to_copy = (mem->size < curl_size) ?
			 mem->size :
			 curl_size;

	memcpy(curl_buf, mem->chunk, to_copy);
	mem->size -= to_copy;
	mem->chunk += to_copy;

	return to_copy;
}
#endif

static int servers_list_progress_func(void *p,
				       curl_off_t dltotal, curl_off_t dlnow,
				       curl_off_t ultotal, curl_off_t ulnow)
{
	progress_t *progress;

	if(p == NULL)
		return 0;

	progress = (progress_t *)p;

	if(dltotal < 1)
		return 0;

	if (progress->done)
		return 0;

	printf("Downloading Server list...%.0f%% \r", (double)(dlnow / (dltotal / 100)));

	if(dltotal == dlnow) {
		progress->done = 1;
		printf("Downloading Server list...done\n");
	}

	fflush(stdout);
	return 0;
}

int get_speedtest_servers_list(const char *file)
{
	int ret = -1;
	progress_t servers_list_download_progress;

	if (!file)
		return -1;

	FILE *fp = fopen(file, "w");

	if (!fp)
		return -1;

	/* 初始化CURL */
	CURL *curl = curl_easy_init();
	CURLcode res = CURLE_OK;

	if (!curl) {
		ret = -1;
		goto exit;
	}

	/*进度条数据 */
	servers_list_download_progress.direction = DIR_DOWNLOAD;
	servers_list_download_progress.lastruntime = 0;
	servers_list_download_progress.last = 0;
	servers_list_download_progress.done = 0;
	servers_list_download_progress.curl = curl;

	/* 设置CURL参数 */
	curl_easy_setopt(curl, CURLOPT_URL, SPEEDTEST_SERVER_URL);/* CURL下载地址 */
	curl_easy_setopt(curl, CURLOPT_WRITEDATA, fp);
	curl_easy_setopt(curl, CURLOPT_USERAGENT, SPEEDTEST_USERAGENT);
	curl_easy_setopt(curl, CURLOPT_FOLLOWLOCATION, 1);
	curl_easy_setopt(curl, CURLOPT_NOPROGRESS, 0);
	curl_easy_setopt(curl, CURLOPT_TIMEOUT, 60L); /* CURL超时时间 */
	curl_easy_setopt(curl, CURLOPT_XFERINFOFUNCTION, servers_list_progress_func);
	curl_easy_setopt(curl, CURLOPT_XFERINFODATA, &servers_list_download_progress);

	curl_easy_setopt(curl, CURLOPT_CONNECTTIMEOUT, 3L);/* 连接超时，这个数值如果设置太短可能导致数据请求不到就断开了 */
	
	res = curl_easy_perform(curl);

	if (res != CURLE_OK)
	{
		fprintf(stdout, "%s\n", curl_easy_strerror(res));
		ret = -1;
		goto exit;
	}
	
	curl_easy_cleanup(curl);
	curl_global_cleanup();
	ret = 0;
exit:
	fclose(fp);
	return ret;
}


static int client_info_progress_func(void *p,
				      curl_off_t dltotal, curl_off_t dlnow,
				      curl_off_t ultotal, curl_off_t ulnow)
{
	progress_t *progress;

	if(p == NULL)
		return 0;

	progress = (progress_t *)p;

	if(dltotal < 1)
		return 0;

	if (progress->done)
		return 0;

	printf("Downloading Client info...%.0f%% \r", (double)(dlnow / (dltotal / 100)));

	if(dltotal == dlnow) {
		progress->done = 1;
		printf("Downloading Client info...done\n");
	}

	fflush(stdout);
	return 0;
}

int get_speedtest_client_info(const char *file)
{

	progress_t client_info_download_progress;

	if (!file)
		return -1;

	FILE *fp = fopen(file, "w");

	if (!fp)
		return -1;

	/* 初始化CURL */
	CURL *curl = curl_easy_init();
	CURLcode res = CURLE_OK;

	if (!curl) {
		fclose(fp);
		return -1;
	}

	/*进度条数据 */
	client_info_download_progress.direction = DIR_DOWNLOAD;
	client_info_download_progress.lastruntime = 0;
	client_info_download_progress.last = 0;
	client_info_download_progress.done = 0;
	client_info_download_progress.curl = curl;

	/* 设置CURL参数 */
	curl_easy_setopt(curl, CURLOPT_URL, SPEEDTEST_CLIENT_URL);/* CURL下载地址 */
	curl_easy_setopt(curl, CURLOPT_WRITEDATA, fp);
	curl_easy_setopt(curl, CURLOPT_USERAGENT, SPEEDTEST_USERAGENT);
	curl_easy_setopt(curl, CURLOPT_FOLLOWLOCATION, 1);
	curl_easy_setopt(curl, CURLOPT_NOPROGRESS, 0);
	curl_easy_setopt(curl, CURLOPT_TIMEOUT, 30L); /* CURL超时时间 */
	curl_easy_setopt(curl, CURLOPT_XFERINFOFUNCTION, client_info_progress_func);
	curl_easy_setopt(curl, CURLOPT_XFERINFODATA, &client_info_download_progress);
	
	curl_easy_setopt(curl, CURLOPT_CONNECTTIMEOUT, 3L);/* 连接超时，这个数值如果设置太短可能导致数据请求不到就断开了 */
	
	res = curl_easy_perform(curl);

	if (res != CURLE_OK)
		fprintf(stdout, "%s\n", curl_easy_strerror(res));

	curl_easy_cleanup(curl);
	curl_global_cleanup();

	fclose(fp);

	return 0;
}

char *make_speedtest_download_url(const char *host, size_t size)
{
	static char url[256] = {0};
	snprintf(url, sizeof(url), "%s%s/%s%ld", "http://", host, "download?size=", (unsigned long)size);
	return url;
}

static int curl_speed_info(void *p,
			    curl_off_t dltotal, curl_off_t dlnow,
			    curl_off_t ultotal, curl_off_t ulnow)
{
	progress_t *progress = (progress_t *)p;
	double curtime = 0;
	double timediff = 0;
	size_t xferdiff = 0;
	double cmbps = 0;
	double ambps = 0;
	curl_off_t total = dltotal;
	curl_off_t now = dlnow;

	if (progress->direction == DIR_UPLOAD) {
		total = ultotal;
		now = ulnow;
	}

	if (total < 1)
		return 0;

	if (progress->done)
		return 0;

	curl_easy_getinfo(progress->curl, CURLINFO_TOTAL_TIME, &curtime);
	timediff = curtime - progress->lastruntime;

	if ((timediff > PROGRESS_INT && now > 0) || now == total) {
		progress->lastruntime = curtime;

		if (progress->direction == DIR_UPLOAD)
			printf("UL");
		else
			printf("DL");

		printf(": %lu of %lu, done: %.0f%%",
			now, total,
			(double)(now / (total / 100)));

		xferdiff = now - progress->last;
		progress->last = now;

		if (xferdiff) {
			cmbps = (double)(((xferdiff / timediff) * 8) / 1000000);
			ambps = (double)(((now / curtime) * 8) / 1000000);
			printf(", current speed: %.2f Mbps, average: %.2f Mbps, Max: %.2f Mbps",
				cmbps, ambps,progress->max_speed);

			progress->avg_speed = ambps;

			if(cmbps > progress->max_speed)
				progress->max_speed = cmbps;
		}

		printf("\r");
		fflush(stdout);

		if (now == total)
			progress->done = 1;
	}

	if (now == total)
		printf("\n");

	return 0;
}

double speedtest_download(const char *host, size_t size)
{
	FILE *fp;
	char *url;
	progress_t download_prog;

	if (!host)
		return -1;

	url = make_speedtest_download_url(host, size);

	//printf("Download form:%s\n", url);

	fp = fopen("/dev/null", "w");

	if (!fp)
		return -1;

	/* 初始化CURL */
	CURL *curl = curl_easy_init();
	CURLcode res = CURLE_OK;

	if (!curl) {
		fclose(fp);
		return -1;
	}

	/* 初始化进度 */
	download_prog.direction = DIR_DOWNLOAD;
	download_prog.lastruntime = 0;
	download_prog.last = 0;
	download_prog.done = 0;
	download_prog.curl = curl;

	curl_easy_setopt(curl, CURLOPT_URL, url);
	curl_easy_setopt(curl, CURLOPT_WRITEDATA, fp);
	curl_easy_setopt(curl, CURLOPT_USERAGENT, SPEEDTEST_USERAGENT);
	curl_easy_setopt(curl, CURLOPT_NOPROGRESS, 0);
	curl_easy_setopt(curl, CURLOPT_NOPROGRESS, 0L);
	curl_easy_setopt(curl, CURLOPT_TIMEOUT, 15L); /* CURL超时时间 */
	curl_easy_setopt(curl, CURLOPT_XFERINFOFUNCTION, curl_speed_info);
	curl_easy_setopt(curl, CURLOPT_XFERINFODATA, &download_prog);
	
	curl_easy_setopt(curl, CURLOPT_CONNECTTIMEOUT, 3L);/* 连接超时，这个数值如果设置太短可能导致数据请求不到就断开了 */
	
	res = curl_easy_perform(curl);

	if (res != CURLE_OK)
	{
		fprintf(stdout, "%s\n", curl_easy_strerror(res));
	}
	
	curl_easy_cleanup(curl);
	curl_global_cleanup();

	fclose(fp);

	//printf("Download speed: %.2f Mbps, average: %.2f Mbps\n", download_prog.max_speed, download_prog.avg_speed);

	return download_prog.avg_speed;
}

double speedtest_upload(const char *url, size_t size)
{
	progress_t upload_prog;

	FILE *null_fp;
	FILE *zero_fp;

	if (!url)
		return -1;

	null_fp = fopen("/dev/null", "w");

	if (!null_fp)
		return -1;

	zero_fp = fopen("/dev/zero", "r");

	if (!zero_fp)
		return -1;

	//printf("Upload to:%s\n", url);

	CURL *curl = curl_easy_init();
	CURLcode res = CURLE_OK;

	if (!curl)
		return -1;

	/* 初始化进度 */
	upload_prog.direction = DIR_UPLOAD;
	upload_prog.lastruntime = 0;
	upload_prog.last = 0;
	upload_prog.done = 0;
	upload_prog.curl = curl;

	curl_easy_setopt(curl, CURLOPT_URL, url);
	curl_easy_setopt(curl, CURLOPT_WRITEDATA, null_fp);
	curl_easy_setopt(curl, CURLOPT_USERAGENT, SPEEDTEST_USERAGENT);
	curl_easy_setopt(curl, CURLOPT_NOPROGRESS, 0L);

	curl_easy_setopt(curl, CURLOPT_UPLOAD, 1L);
	curl_easy_setopt(curl, CURLOPT_PUT, 1L);
	curl_easy_setopt(curl, CURLOPT_URL, url);

	curl_easy_setopt(curl, CURLOPT_READDATA, zero_fp);
	curl_easy_setopt(curl, CURLOPT_INFILESIZE_LARGE,
			  (curl_off_t)size);

	curl_easy_setopt(curl, CURLOPT_TIMEOUT, 20L); /* CURL超时时间 */
	curl_easy_setopt(curl, CURLOPT_XFERINFOFUNCTION, curl_speed_info);
	curl_easy_setopt(curl, CURLOPT_XFERINFODATA, &upload_prog);

	curl_easy_setopt(curl, CURLOPT_CONNECTTIMEOUT, 3L);/* 连接超时，这个数值如果设置太短可能导致数据请求不到就断开了 */
	
	res = curl_easy_perform(curl);

	if (res != CURLE_OK)
		fprintf(stdout, "%s\n", curl_easy_strerror(res));

	curl_easy_cleanup(curl);
	curl_global_cleanup();

	fclose(null_fp);

	//printf("Upload speed: %.2f Mbps, average: %.2f Mbps\n", upload_prog.max_speed, upload_prog.avg_speed);

	return upload_prog.avg_speed;
}
