#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include <string.h>
#include <libxml/parser.h>
#include <libxml/tree.h>
#include <curl/curl.h>
#include <sys/stat.h>

#include "speedtest.h"

// strdup isnt a C99 function, so we need it to define itself
char *strdup(const char *str)
{
	int n = strlen(str) + 1;
	char *dup = malloc(n);

	if(dup) {
		strcpy(dup, str);
	}

	return dup;
}

int sort_servers(speedtest_server_t **srv1, speedtest_server_t **srv2)
{
	return((*srv1)->distance - (*srv2)->distance);
}

long have_sine_distance(float lat1, float lon1, float lat2, float lon2)
{
	float dx, dy, dz, a, b;
	lon1 -= lon2;
	lon1 *= TO_RAD, lat1 *= TO_RAD, lat2 *= TO_RAD;

	dz = sin(lat1) - sin(lat2);
	dx = cos(lon1) * cos(lat1) - cos(lat2);
	dy = sin(lon1) * cos(lat1);
	a = (dx * dx + dy * dy + dz * dz);
	b = sqrt(a) / 2;
#ifdef USE_ASIN
	return 2 * R * asin(b);
#else
	return 2 * R * atan2(b, sqrt(1 - b * b));
#endif
}

speedtest_server_t *get_best_server_by_distance(speedtest_client_t *client)
{
	int i;

	for(i = 0; i < server_count; i++) {
		server_list[i]->distance = have_sine_distance(client->lat,
					   client->lon,
					   server_list[i]->lat,
					   server_list[i]->lon);
		//printf("[%f,%f]->[%f,%f]Dist: %ld km\n",client->lat, client->lon,server_list[i]->lat, server_list[i]->lon, server_list[i]->distance);
	}

	qsort(server_list, server_count, sizeof(speedtest_server_t *),
	      (int (*)(const void *, const void *)) sort_servers);

	return server_list[0];
}
