#include "speedtest.h"

xmlNode *find_node(xmlNode *parent, const char *name )
{
	xmlNode *iter = NULL;

	if (!parent || !name )
		return NULL;

	for (iter = parent->children; iter; iter = iter->next ){
		if (iter->type != XML_ELEMENT_NODE )
			continue;

		if (!strncmp(iter->name, name, strlen(name)+ 1 ))
			return iter;
	}

	return NULL;
}

xmlNode *find_next_node(xmlNode *prev, const char *name )
{
	xmlNode *iter = NULL;

	if (!prev || !name )
		return NULL;

	for (iter = prev->next; iter; iter = iter->next ){
		if (iter->type != XML_ELEMENT_NODE )
			continue;

		if (!strncmp(iter->name, name, strlen(name)+ 1 ))
			return iter;
	}

	return NULL;
}

xmlChar *get_node_attribute(xmlNode *node, const char *name )
{
	xmlAttr *attr  = NULL;

	if (!node || !name )
		return NULL;

	attr = node->properties;

	while(attr && attr->name && attr->children ){
		if (strncmp(attr->name, name, strlen(name)+ 1 )){
			attr = attr->next;
			continue;
		}

		return xmlNodeListGetString(node->doc, attr->children, 1);
	}

	return NULL;
}

xmlChar *get_node_content(xmlNode *node )
{
	if (!node || !node->children )
		return NULL;

	if (node->children->content )
		return strdup(node->children->content);

	return NULL;
}

int convert_server_xml(char *xml_file)
{
	xmlDoc *xml = NULL;
	xmlNode *root = NULL;

	xmlChar *url = NULL;
	xmlChar *url2 = NULL;
	xmlChar *host = NULL;
	xmlChar *country = NULL;
	xmlChar *city = NULL;
	xmlChar *sponsor = NULL;
	xmlChar *lon = NULL;
	xmlChar *lat = NULL;
	xmlChar *id = NULL;

	xmlNode *current_node = NULL;

	int ret = -1;

	int i = 0;

	/* initialize xml library */
	LIBXML_TEST_VERSION

	xml = xmlReadFile(xml_file, NULL, 0);

	if (!xml ){
		printf("could not parse server list %s\n", xml_file);
		goto exit;
	}

	/* root element node */
	root = xmlDocGetRootElement(xml);

	server_count = 0;
	/* FIXME:only get the server list*/
	current_node = find_node(root, "servers");

	for (current_node = find_node(current_node, "server");
	      current_node;
	      current_node = find_next_node(current_node, "server" )){
		server_count++;
	}

	server_list = (speedtest_server_t **)malloc(sizeof(speedtest_server_t)* server_count);

	if(server_list == NULL)
		return -1;

	current_node = find_node(root, "servers");

	for (current_node = find_node(current_node, "server");
	      current_node;
	      current_node = find_next_node(current_node, "server" )){
		country = get_node_attribute(current_node, "country");
		city = get_node_attribute(current_node, "name");
		sponsor = get_node_attribute(current_node, "sponsor");
		url = get_node_attribute(current_node, "url");
		host = get_node_attribute(current_node, "host");
		lon = get_node_attribute(current_node, "lon");
		lat = get_node_attribute(current_node, "lat");
		id = get_node_attribute(current_node, "id");

		/* 必须是存在下面的数值才有效 */
		if (city && sponsor && country && url){

			server_list[i] = malloc(sizeof(speedtest_server_t));

			if(server_list[i] == NULL)
				continue;

			server_list[i]->url = url;
			server_list[i]->country = country;
			server_list[i]->host = host;
			server_list[i]->id = id;
			server_list[i]->lon = atof(lon);
			server_list[i]->lat = atof(lat);
			server_list[i]->sponsor = sponsor;
			server_list[i]->name = city;
			//printf("url:%s lon:%f lat:%f\n",server_list[i]->url,server_list[i]->lon,server_list[i]->lat);
			i++;
		}
	}

	printf("Got %d Speedtest.net servers \n", server_count);

	ret = server_count;

exit:
	return ret;
}

speedtest_client_t *convert_client_xml(char *xml_file)
{
	xmlDoc *xml = NULL;
	xmlNode *root = NULL;

	xmlChar *country = NULL;
	xmlChar *ip = NULL;
	xmlChar *isp = NULL;
	xmlChar *lon = NULL;
	xmlChar *lat = NULL;

	xmlNode *current_node = NULL;

	speedtest_client_t *client = NULL;

	/* initialize xml library */
	LIBXML_TEST_VERSION

	xml = xmlReadFile(xml_file, NULL, 0);

	if (!xml ){
		printf("could not parse server list %s\n", xml_file);
		goto exit;
	}

	/* root element node */
	root = xmlDocGetRootElement(xml);

	current_node = find_node(root, "client");

	ip = get_node_attribute(current_node, "ip");
	isp = get_node_attribute(current_node, "isp");
	lat = get_node_attribute(current_node, "lat");
	lon = get_node_attribute(current_node, "lon");
	country = get_node_attribute(current_node, "country");

	if(ip){
		client = (speedtest_client_t *)malloc(sizeof(speedtest_client_t));

		if(client == NULL)
			goto exit;

		client->ip = ip;
		client->isp = isp;
		client->lat = atof(lat);
		client->lon = atof(lon);
		//printf("IP:%s ISP:%s [%f,%f]\n",client->ip,client->isp,client->lat,client->lon);
	}

exit:
	return client;
}
